uniform Materials {
    highp uvec4 materials[NUM_OBJECTS*MATERIAL_SIZE + 1u];
    highp uvec4 alphaMaterials[(NUM_OBJECTS >> 1u) + 1u];
};

lowp uvec4 uintToVec4u(highp uint val) {
    return (uvec4(val) >> uvec4(0u, 8u, 16u, 24u)) & uvec4(0xFFu);
}

/** Decode a color encoded into 32 bit */
lowp vec4 uintToColor4ub(highp uint val) {
    return vec4(uintToVec4u(val))*(1.0/255.0);
}

lowp uint byteToUint(highp uint val, lowp uint b) {
    return (val >> b*8u) & 0xFFu;
}

lowp float byteToFloat(highp uint val, lowp uint b) {
    return float(byteToUint(val, b))*(1.0/255.0);
}

mediump uint uvec2ToUint(lowp uvec2 val) {
    /* Promote higher to cast expression to mediump
     * and avoid overflow with the bit-shift */
    mediump uint higher = val.y;
    return (higher << 8u) | val.x;
}

#ifdef TEXTURED
void alphaMask(uint objectId, vec2 textureCoords) {
#ifdef ALPHA
#ifdef ALPHA_MASKED
    uvec4 group = alphaMaterials[objectId >> 1u];
    uvec2 alphaMat = (objectId & 1u) == 0u ? group.xy : group.zw;
    uint alphaMaskTexture = alphaMat.x;

    if(alphaMaskTexture == 0u) return;

    float alpha = textureAtlas(alphaMaskTexture, textureCoords).a;
    float alphaMaskThreshold = uintBitsToFloat(alphaMat.y);
    if(alpha < alphaMaskThreshold) discard;
#endif
#endif
}
#endif
