
/* Texture atlasses: 0 = uncompressed, 1 = compressed */
uniform lowp sampler2DArray textures[2];

uniform mediump usampler2D indirectionTexture;
#define TEXTURE_TILE_W 256
#define TEXTURE_TILE_H 256

uniform Textures {
    highp vec4 textureBounds[NUM_TEXTURES];
    highp uvec4 textureLayer[NUM_TEXTURES >> 2];
};

vec4 tileStack(lowp sampler2DArray stack, vec2 uv, int lod) {
    vec2 indirectionTextureSize = vec2(textureSize(indirectionTexture, 0));
    uint tile = texelFetch(indirectionTexture, ivec2(uv*indirectionTextureSize), 0).r;
    /* Compute UVs relative to the tile */
    ivec2 tileUV = ivec2(
        fract(uv*indirectionTextureSize)
            #ifdef MIP_MAPPING
            *float(TEXTURE_TILE_H >> lod)
            #else
            *float(TEXTURE_TILE_H)
            #endif
    );
    return texelFetch(stack, ivec3(tileUV, tile), lod);
}

#ifdef MIP_MAPPING
lowp int mipMapLevel(vec2 pixelCoords) {
    highp vec2 dx = dFdx(pixelCoords);
    highp vec2 dy = dFdy(pixelCoords);
    highp float d = max(dot(dx, dx), dot(dy, dy));

    /* Clamp to valid mip range */
    return int(clamp(float(floor(log2(sqrt(d)))), 0.0, 7.0));
}
#endif

uint vecComp(uvec4 v, uint c) {
    return (c == 0u) ? v.x :
        (c == 1u) ? v.y :
        (c == 2u) ? v.z : v.w;
}

vec4 textureAtlas(lowp uint t, vec2 uv) {
    /* The single uints are stored in UBO for more straight forward packing.
     * We divide by 4 and modulo 4 to get the correct vector and component
     * of vector */
    uint layer = vecComp(textureLayer[t >> 2u], t & 3u);

    /* Get the bounds on the texture altas */
    vec4 bounds = textureBounds[t];

    /* Whether the texture is compressed or not is stored in the final bit */
    if((layer & 1u) == 0u) {

        vec3 finalUV = vec3(bounds.xy + fract(uv)*bounds.zw, layer >> 1u);
        return texture(textures[0], finalUV);
    } else {
        /* Get the bounds on the texture altas */
        vec2 vTextureSize = vec2(textureSize(indirectionTexture, 0))*float(TEXTURE_TILE_H);
        highp vec2 uvToTexelSize = bounds.zw*vTextureSize;

#ifdef MIP_MAPPING
        int mip = mipMapLevel(uv*uvToTexelSize);
        /* Explicit cast to int is required to avoid shader compilation issues in the browser! */
        vec2 uvToTexelSizeMipped = uvToTexelSize/float(int(1 << mip));
#else
        const int mip = 0;
        vec2 uvToTexelSizeMipped = uvToTexelSize;
#endif

        vec2 texelSizeToUv = 1.0/uvToTexelSizeMipped;
        /* Bilinear filtering */
        vec2 f = fract(uv*uvToTexelSizeMipped);
        uv += (0.5 - f)*texelSizeToUv;

        vec4 tl = tileStack(textures[1], bounds.xy + fract(uv)*bounds.zw, mip);
        vec4 tr = tileStack(textures[1], bounds.xy + fract(uv + vec2(texelSizeToUv.x, 0.0))*bounds.zw, mip);
        vec4 tA = mix(tl, tr, f.x);

        vec4 bl = tileStack(textures[1], bounds.xy + fract(uv + vec2(0.0, texelSizeToUv.y))*bounds.zw, mip);
        vec4 br = tileStack(textures[1], bounds.xy + fract(uv + texelSizeToUv)*bounds.zw, mip);
        vec4 tB = mix(bl, br, f.x);
        return mix(tA, tB, f.y);
    }
}
